/*********************************************************************
							led_mat.c
							=========
						Uwe Berger; 2008
						
	* BLIT2008-Board (http://bralug.de/wiki/BLIT2008-Board)
	* Ansteuerung einer 4x4-LED-Matrix via TWI (PCF8574)						

*********************************************************************/



#ifndef F_CPU
	#define F_CPU 12000000
#endif

#include <avr/interrupt.h>
#include "i2clcd.h"

#define ADDR_TASTMAT 0x42
#define ADDR_LEDMAT  0x44

#define PRELOAD		65487					// Preload-Wert des Timers (240Hz)

unsigned char zeile = 1;

// 2 Zaehlervariablen
struct	{
			unsigned char z1 : 4;
			unsigned char z2 : 4;
		} z;
	
// Definition der Variable, die die LED-Zustaende aufnimmt
volatile union u_led	{
					struct	{
								unsigned int l1 : 1;
								unsigned int l2 : 1;
								unsigned int l3 : 1;
								unsigned int l4 : 1;
								unsigned int l5 : 1;
								unsigned int l6 : 1;
								unsigned int l7 : 1;
								unsigned int l8 : 1;
								unsigned int l9 : 1;
								unsigned int l10 : 1;
								unsigned int l11 : 1;
								unsigned int l12 : 1;
								unsigned int l13 : 1;
								unsigned int l14 : 1;
								unsigned int l15 : 1;
								unsigned int l16 : 1;
							} l;
					struct	{
								unsigned int s1 : 4;
								unsigned int s2 : 4;
								unsigned int s3 : 4;
								unsigned int s4 : 4;
							} s;
					unsigned int all;
				} led;						

/* -------------------------------------------------------------------------- */
unsigned char keyboard_led_write(unsigned char val)
{
	if (i2c_start(ADDR_LEDMAT+I2C_WRITE)) return 1;
	i2c_write(val);
	i2c_stop();
	return 0;
}

/* -------------------------------------------------------------------------- */
ISR(TIMER1_OVF_vect)
{
	TCNT1     = PRELOAD;
	zeile *= 2;
	if (zeile > 15) zeile = 1;
	// an dieser Stelle wird ein switch genommen, um fuer alle
	// Spalten das gleiche Zeitverhalten zu haben (bei einer 
	// if-Orgie ist das nicht gewaehrleistet!)
	switch (zeile)
		{
			case 1: keyboard_led_write(~(led.s.s1*16 + zeile)); break;
			case 2: keyboard_led_write(~(led.s.s2*16 + zeile)); break;
			case 4: keyboard_led_write(~(led.s.s3*16 + zeile)); break;
			case 8:	keyboard_led_write(~(led.s.s4*16 + zeile)); break;
		}
}


/* -------------------------------------------------------------------------- */
void long_delay(uint16_t ms) {				// lange Pause
    for (; ms>0; ms--) _delay_ms(1);
}


/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
/* -------------------------------------------------------------------------- */
int main(void)
{
	unsigned char x;
	
    // Timer-Overflow-Interrupt
    TCNT1   = PRELOAD;						// Preloadwert setzen
	TCCR1B |= (1 << CS02) | (1 << CS00);	// Prescaler Timer1 auf 1024 stellen
	TIMSK  |= (1 << TOIE1);					// Timer 1 Overflow Interrupt enable

	sei(); 									// Interrupts einschalten
	
	i2c_init();								// TWI initialisieren
	led.all = 0;							// alle LEDs aus

	z.z1 = 0;								// Zaehler vorinitialisieren
	z.z2 = 1;

	while (1) {		

		// "Bilder" neu setzen
		led.all = 0;
		led.s.s1 =  z.z1;					// binaerer 4-Bit-Zahler
		led.s.s2 = ~z.z1;					// der invertierte Wert dazu
		led.s.s3 = z.z2;					// "Lauflicht"
		led.s.s4 = ~z.z2;					// der invertierte Wert dazu
		
		// interne Zaehler weiter
		z.z1++;								// z.z1 ist eine 4-Bit-Variable...
		z.z2 *= 2;							// 1 weiterschieben...
		if (!z.z2) z.z2 = 1;				// 1 ist aus den 4 Bit rausgeschoben...
		
		long_delay(400);


	}

	return 0;
}
